"""paver config file"""

from paver.easy import sh
from paver.tasks import task, needs


@task
def pytest():
    """unit testing"""
    sh('pytest')

@task
def pylint():
    """pyltin"""
    sh('pylint ./drupal_download/')

@task
def pypi():
    """Instalation on PyPi"""
    sh('python setup.py sdist')
    sh('twine upload dist/*')

@task
def local():
    """local install"""
    sh("pip uninstall drupal_download")
    sh("python setup.py install develop")


@task
def sphinx():
    """Document creation using Shinx"""
    sh('cd docs; make html; cd ..')

@needs('pytest', 'pylint', 'sphinx')
@task
def default():
    """default"""
    pass



import json
from typing import Dict

import pytest
import requests
import requests_mock

from drupal_download.downloader import AuthType, DrupalDownloadException, Drupal7DadaDownloader, Drupal8DadaDownloader


class TestDownloader:
    website = "https://www.example.com"
    base_url = website + "/export"

    def test_authentication_anonymous(self):
        with requests_mock.Mocker() as mock:
            mock.get(self.base_url, text='[]')
            dl = Drupal7DadaDownloader(self.base_url, None, None, AuthType.Anonymous, lambda x: x)
            dl.load_data()
            assert dl.pages_count == 0
            assert dl.objects_count == 0

    def test_authentication_session(self):
        with requests_mock.Mocker() as mock:
            mock.get(self.base_url, text='[]')
            mock.post(self.website + "/user?_format=json", text='')

            dl = Drupal7DadaDownloader(self.base_url, "john", "123", AuthType.CookieSession, lambda x: x)
            dl.load_data()
            assert dl.pages_count == 0
            assert dl.objects_count == 0

    def test_authentication_session_fails(self):
        with requests_mock.Mocker() as mock:
            mock.post(self.website + "/user?_format=json", text='', status_code=403)

            dl = Drupal7DadaDownloader(self.base_url, "john", "123", AuthType.CookieSession, lambda x: x)
            with pytest.raises(DrupalDownloadException):
                dl.load_data()

    def test_authentication_basic(self):
        with requests_mock.Mocker() as mock:
            mock.get(self.base_url, text='[]')

            dl = Drupal7DadaDownloader(self.base_url, "john", "123", AuthType.HTTPBasic, lambda x: x)
            dl.load_data()
            assert dl.pages_count == 0
            assert dl.objects_count == 0

    def validate_object_id(self, data: Dict, expected: int):
        dl = Drupal7DadaDownloader(self.base_url, None, None, AuthType.Anonymous, None)
        assert dl.get_object_id(data) == expected

    def test_get_object_id(self):
        self.validate_object_id(dict(cid="2", nid="30"), 2)
        self.validate_object_id(dict(tid="2", nid="30"), 2)
        self.validate_object_id(dict(vid="2", nid="30"), 2)
        self.validate_object_id(dict(xxtid="2", nid="30"), 30)

    def test_get_object_id_captures_id_name(self):
        dl = Drupal7DadaDownloader(self.base_url, None, None, AuthType.Anonymous, None)
        data = dict(cid="2", nid="30")
        assert dl.get_object_id(data) == 2
        assert dl.get_object_id(dict(cid=70)) == 70

    def test_get_object_id_drupal8(self):
        dl = Drupal8DadaDownloader(self.base_url, None, None, AuthType.Anonymous, None, id_name="nid")
        data = dict(cid="2", nid=[{"value": "30"}])
        assert dl.get_object_id(data) == 30

    def test_download(self):
        count = 0

        def incr(x):
            nonlocal count
            count += 1

        data = [dict(nid=i, uri=self.base_url + f"?node={i}") for i in range(20)]
        with requests_mock.Mocker() as mock:
            mock.register_uri('GET', url=self.base_url, text=json.dumps(data), complete_qs=True)
            mock.register_uri('GET', url=self.base_url + "?page=1", text="[]", complete_qs=True)
            mock.post(self.website + "/user?_format=json", text='')

            dls = [Drupal8DadaDownloader(self.base_url, None, None, AuthType.Anonymous, incr, id_name="nid")]
            for dl in dls:
                count = 0
                dl.load_data()
                assert dl.pages_count == 1
                assert dl.objects_count == 20
                assert count == 20

    def test_download_drupal7(self):
        count = 0

        def incr(x):
            nonlocal count
            count += 1

        data = [dict(nid=i, uri=self.base_url + f"?node={i}") for i in range(20)]
        with requests_mock.Mocker() as mock:
            mock.register_uri('GET', url=self.base_url, text=json.dumps(data), complete_qs=True)
            mock.register_uri('GET', url=self.base_url + "?page=1", text="[]", complete_qs=True)
            for node in data:
                mock.register_uri('GET', url=self.base_url + "?node=" + str(node["nid"]),
                                  text=json.dumps(node), complete_qs=True)
            mock.post(self.website + "/user?_format=json", text='')
            mock.post(self.website + "/user/login?_format=json", text='')

            dls = [Drupal8DadaDownloader(self.base_url, "john", "123", AuthType.CookieSession, incr, id_name="nid"),
                   Drupal7DadaDownloader(self.base_url, None, None, AuthType.Anonymous, incr)]
            for dl in dls:
                count = 0
                dl.load_data()
                assert dl.pages_count == 1
                assert dl.objects_count == 20
                assert count == 20



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
from builtins import Exception
from enum import Enum, auto

from typing import Iterable, Callable, Set, Optional

import logging

import requests
from requests.auth import HTTPBasicAuth

from furl import furl


class DrupalDownloadException(Exception):
    """
    The exception this library uses to report problems
    """
    pass


class AuthType(Enum):
    """
    Authentication types for Drupal REST API
    """
    Anonymous = auto()
    HTTPBasic = auto()
    CookieSession = auto()


class DrupalDadaDownloader(object):
    """
    Download data from a Drupal REST API endpoint. This specific class assumes that each page carries all the
    information needed for each individual object, so it doesn't need to download anything else.
    """

    logger = logging.getLogger("DrupalDownload")
    ids_sequence = ["cid", "tid", "vid", "nid"]

    def __init__(self,
                 base_url: str,
                 username: str,
                 password: str,
                 auth_type: AuthType,
                 on_object: Callable,
                 page_size: Optional[int] = None,
                 skip_duplicates: bool = True,
                 id_name: Optional[str] = None):
        """
        Constructor of the class

        :param base_url: the base url of the API endpoint
        :param username: user for the API access. None if anonymous access is available
        :param password: password for the API access. None if anonymous access is available
        :param auth_type: authentication type, as defined in the endpoint configuration
        :param on_object: a callback to process each retrieved object
        :param page_size: how many items retrieve per page
        :param skip_duplicates: whether to skip the same data if seen more than once
        """
        super().__init__()
        self.initial_url: furl = furl(base_url)
        self.username = username
        self.password = password
        self.auth_type = auth_type
        self.on_object = on_object
        self.page_size = page_size
        self.skip_duplicates = skip_duplicates
        self.id_name: Optional[str] = id_name

        if self.page_size is not None and self.page_size <= 0:
            raise DrupalDownloadException(f"Page size must be positive or None")

        self.objects_count = 0
        self.pages_count = 0
        self.logged_in = False
        self.session = requests.Session()
        self.auth = None
        self.seen_objects: Set[int] = set()

    def get_url(self, url: str) -> requests.Response:
        if self.auth_type == AuthType.HTTPBasic:
            if self.auth is None:
                self.auth = HTTPBasicAuth(self.username, self.password)
            return self.session.get(url, auth=self.auth)
        elif self.auth_type == AuthType.Anonymous:
            return self.session.get(url)
        elif self.auth_type == AuthType.CookieSession:
            if not self.logged_in:
                self.logger.info(f"Not logged in, login into Drupal session for {self.username}")
                login = dict()
                login['name'] = self.username
                login['pass'] = self.password
                login_url = self.get_login_url()
                headers = {'Content-type': 'application/json'}
                response = self.session.post(login_url.url, json=login, headers=headers)
                if not response.ok:
                    raise DrupalDownloadException(f"Failed to login: {response.reason}")
                self.logged_in = True
            return self.session.get(url)
        else:
            raise DrupalDownloadException(f"Unsupported authentication type {self.auth_type}")

    def load_data(self):
        """
        Load the objects from the API endpoint. On each valid object the on_object callback is invoked.
        """
        self.objects_count = 0
        self.pages_count = 0
        load_next = True
        page = -1
        url = self.initial_url.copy()
        self.logger.info(f"Starting data extraction from {self.initial_url}")
        while load_next:
            page += 1
            if page > 0:
                url.args["page"] = page
            elif "page" in url.args:
                del url.args["page"]
            if self.page_size is not None:
                url.args["pagesize"] = self.page_size
            response = self.get_url(url.url)
            if not response.ok:
                raise DrupalDownloadException(f"Failed to get the data from {url}: {response.reason}")
            if response.encoding is None:
                response.encoding = "utf-8"
            data = response.json()

            count = 0
            if self.skip_duplicates:
                filtered_data = self.filter_out_duplicates(data)
            else:
                filtered_data = data
            for obj in self.page_to_objects(filtered_data):
                obj_id = self.get_object_id(obj)
                self.seen_objects.add(obj_id)
                count += 1
                self.objects_count += 1
                self.process_object(obj)

            if len(data) > 0:
                self.pages_count += 1
                self.logger.debug(f"Loaded page {page + 1} with {count:,} objects on it")
            else:
                load_next = False
        self.logger.info(
            f"Finished data extraction from {self.initial_url}; pages={self.pages_count:,}, objects={self.objects_count:,}")

    def page_to_objects(self, page_data) -> Iterable:
        pass

    def process_object(self, obj):
        self.on_object(obj)

    def get_object_id(self, obj) -> int:
        pass

    def get_login_url(self) -> furl:
        pass

    def page_to_objects_list(self, page_data) -> Iterable:
        for obj_data in page_data:
            url = obj_data["uri"]
            response = self.get_url(url)
            if not response.ok:
                raise DrupalDownloadException(f"Failed to get the data for node from {url}: {response.reason}")
            yield response.json()

    def filter_out_duplicates(self, data) -> Iterable:
        return [n for n in data if self.get_object_id(n) not in self.seen_objects]


class Drupal7DadaDownloader(DrupalDadaDownloader):
    """
    A refinement of the downloader class, applicable to the standard service REST APIs in Drupal. This module
    implementation provides only basic data on the index page. To get full data, this class uses the provided uri
    attribute for each individual object.
    """

    def page_to_objects(self, page_data) -> Iterable:
        return self.page_to_objects_list(page_data)

    def get_login_url(self):
        res = self.initial_url.copy().set(path="user")
        if "_format" not in res.args:
            return res.add(dict(_format="json"))
        else:
            return res

    def get_object_id(self, obj) -> int:
        if self.id_name is None:
            for id_name in self.ids_sequence:
                if id_name in obj:
                    self.id_name = id_name
                    break
            if self.id_name is None:
                raise DrupalDownloadException("Unable to find an object id")
        return int(obj[self.id_name])


class Drupal8DadaDownloader(DrupalDadaDownloader):
    """
    A refinement of the downloader class, applicable to Drupal 8. In this version you must use the built in REST
    services, which will expose all the data on one page, without the need to refer to additional resources.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.id_name is None:
            raise DrupalDownloadException("id_name is mandatory for Drupal 8")

    def page_to_objects(self, page_data) -> Iterable:
        return page_data

    def get_login_url(self):
        res = self.initial_url.copy().set(path="user/login")
        if "_format" in res.args:
            del res.args["_format"]
        return res.add(dict(_format="json"))

    def get_object_id(self, obj) -> int:
        res = obj[self.id_name]
        if isinstance(res, list):
            res = res[0]["value"]
        return int(res)



import setuptools



def readme():
    with open('README.rst') as f:
        return f.read()


setuptools.setup(name='drupal_download',
                 version='0.0.6',
                 description='Download data from Drupal using Python',
                 long_description=readme(),
                 classifiers=[
                     'Development Status :: 3 - Alpha',
                     'License :: OSI Approved :: MIT License',
                     'Programming Language :: Python :: 3',
                     'Topic :: Internet',
                 ],
                 keywords='drupal download data migration wagtail',
                 url='https://github.com/zelenij/python-drupal-download',
                 author="Andre Bar'yudin",
                 author_email='andre.baryudin@gmail.com',
                 license='MIT',
                 packages=setuptools.find_packages(),
                 include_package_data=True,
                 python_requires='>=3.6',
                 install_requires=["requests", "python-dateutil", "furl", "configargparse"],
                 tests_require=['requests_mock', 'pytest'],
                 zip_safe=False)



import json
import sys
import io

import datetime as dt
import configargparse
import logging

import dateutil.parser as dp
from furl import furl

from drupal_download.downloader import AuthType, DrupalDownloadException, Drupal7DadaDownloader, Drupal8DadaDownloader


def download_main():
    logging.getLogger('urllib3').setLevel(logging.INFO)
    logging.getLogger('requests').setLevel(logging.INFO)
    log = logging.getLogger('')
    log.setLevel(logging.DEBUG)
    format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)-4s - %(message)s")
    ch = logging.StreamHandler(sys.stdout)
    ch.setFormatter(format)
    log.addHandler(ch)

    data = list()

    def collector(obj):
        nonlocal data
        data.append(obj)

    parser = configargparse.ArgumentParser(description='Drupal download dump utility. Download data from a Drupal site'
                                                       ' and write it to a json file')
    parser.add_argument('-c', '--config', required=False, is_config_file=True, help='config file path')
    parser.add_argument('-b', '--base-url',
                        help='Base url for the Drupal site')
    parser.add_argument('--auth-type', choices=[x.name for x in AuthType],
                        help='The authentication type')
    parser.add_argument('--username', required=False,
                        help='The user name')
    parser.add_argument('--password', required=False,
                        help='Password')
    parser.add_argument('-o', '--output',
                        help='Output file')
    parser.add_argument('--drupal-version', choices=[7, 8], type=int,
                        help='Drupal version')
    parser.add_argument('--page-size', type=int,
                        help='Page size')
    parser.add_argument('--id-name', required=False,
                        help='ID name of the data being downloaded. For node it will be nid, for comment cid etc.')

    args = parser.parse_args()
    auth_type = [x for x in AuthType if x.name == args.auth_type][0]
    if auth_type != AuthType.Anonymous:
        if args.username is None or args.password is None:
            raise DrupalDownloadException(f"Username and password required for auth type {auth_type}")

    if args.drupal_version == 7:
        downloader = Drupal7DadaDownloader
    elif args.drupal_version == 8:
        downloader = Drupal8DadaDownloader
    else:
        raise DrupalDownloadException(f"Unsupported Drupal version {args.drupal_version}")
    dl = downloader(args.base_url,
                    args.username,
                    args.password,
                    auth_type,
                    collector,
                    page_size=args.page_size,
                    id_name=args.id_name)
    dl.load_data()

    with io.open(args.output, 'w', encoding='utf-8') as fs:
        json.dump(data, fs, ensure_ascii=False, indent=4)
